"""

  This file keeps all fixes for issues found

"""
import os
import datetime
from textwrap import dedent
from unittest import TestCase

import pyexcel as pe
from pyexcel_libxlsxw import save_data

from nose.tools import eq_


class TestBugFix(TestCase):
    def setUp(self):
        self.maxDiff = None

    def test_pyexcel_issue_5(self):
        """pyexcel issue #5

        datetime is not properly parsed
        """
        s = pe.load(
            os.path.join("tests", "test-fixtures", "test-date-format.xls")
        )
        s.save_as("issue5.xlsx", library="pyexcel-libxlsxw")
        s2 = pe.load("issue5.xlsx")
        eq_(s[0, 0], datetime.datetime(2015, 11, 11, 11, 12, 0))
        eq_(s2[0, 0], datetime.datetime(2015, 11, 11, 11, 12, 0))

    def test_pyexcel_issue_8_with_physical_file(self):
        """pyexcel issue #8

        formular got lost
        """
        tmp_file = "issue_8_save_as.xlsx"
        s = pe.load(os.path.join("tests", "test-fixtures", "test8.xlsx"))
        s.save_as(tmp_file, library="pyexcel-libxlsxw")
        s2 = pe.load(tmp_file)
        self.assertEqual(str(s), str(s2))
        content = dedent(
            """
        CNY:
        +----------+----------+------+---+-------+
        | 01/09/13 | 02/09/13 | 1000 | 5 | 13.89 |
        +----------+----------+------+---+-------+
        | 02/09/13 | 03/09/13 | 2000 | 6 | 33.33 |
        +----------+----------+------+---+-------+
        | 03/09/13 | 04/09/13 | 3000 | 7 | 58.33 |
        +----------+----------+------+---+-------+"""
        ).strip("\n")
        self.assertEqual(str(s2), content)
        os.unlink(tmp_file)

    def test_pyexcel_issue_8_with_memory_file(self):
        """pyexcel issue #8

        formular got lost
        """
        tmp_file = "issue_8_save_as.xlsx"
        f = open(os.path.join("tests", "test-fixtures", "test8.xlsx"), "rb")
        s = pe.load_from_memory("xlsx", f.read())
        s.save_as(tmp_file, library="pyexcel-libxlsxw")
        s2 = pe.load(tmp_file)
        self.assertEqual(str(s), str(s2))
        content = dedent(
            """
        CNY:
        +----------+----------+------+---+-------+
        | 01/09/13 | 02/09/13 | 1000 | 5 | 13.89 |
        +----------+----------+------+---+-------+
        | 02/09/13 | 03/09/13 | 2000 | 6 | 33.33 |
        +----------+----------+------+---+-------+
        | 03/09/13 | 04/09/13 | 3000 | 7 | 58.33 |
        +----------+----------+------+---+-------+"""
        ).strip("\n")
        self.assertEqual(str(s2), content)
        os.unlink(tmp_file)

    def test_excessive_columns(self):
        tmp_file = "date_field.xlsx"
        s = pe.get_sheet(file_name=os.path.join("tests", "fixtures", tmp_file))
        assert s.number_of_columns() == 2

    def test_workbook_options(self):
        cell_content = "= Hello World ="
        tmp_file = "workbook_options.xlsx"
        data = {"Sheet 1": [[cell_content]]}
        save_data(
            tmp_file,
            data,
            strings_to_formulas=False,
            library="pyexcel-libxlsxw",
        )
        sheet = pe.get_sheet(file_name=tmp_file)
        self.assertEqual(sheet[0][0], cell_content)
        os.unlink(tmp_file)



"""
    pyexcel_libxlsxw
    ~~~~~~~~~~~~~~~~~~~

    The lower level xlsx file format writer using libxlsxwpy

    :copyright: (c) 2020 by Onni Software Ltd & its contributors
    :license: New BSD License
"""
from pyexcel_io.io import get_data as read_data
from pyexcel_io.io import isstream
from pyexcel_io.io import store_data as write_data

# flake8: noqa
# this line has to be place above all else
# because of dynamic import
from pyexcel_io.plugins import IOPluginInfoChainV2

__FILE_TYPE__ = "xlsx"
IOPluginInfoChainV2(__name__).add_a_writer(
    relative_plugin_class_path="xlsxw.XLSXWriter",
    locations=["file"],
    file_types=[__FILE_TYPE__],
    stream_type="binary",
)


def save_data(afile, data, file_type=None, **keywords):
    """standalone module function for writing module supported file type"""
    if isstream(afile) and file_type is None:
        file_type = __FILE_TYPE__
    write_data(afile, data, file_type=file_type, **keywords)



"""
    pyexcel_libxlsxw
    ~~~~~~~~~~~~~~~~~~~

    The lower level xlsx file format writer using libxlsxwpy

    :copyright: (c) 2016-2020 by Onni Software Ltd & its contributors
    :license: New BSD License
"""
from datetime import date, time, datetime

from libxlsxwpy import Book
from pyexcel_io.plugin_api import IWriter, ISheetWriter


class XLSXSheetWriter(ISheetWriter):
    """
    xlsx sheet writer
    """

    def __init__(self, xlsx_sheet):
        self.xlsx_sheet = xlsx_sheet
        self.current_row = 0

    def write_row(self, array):
        """
        write a row into the file
        """
        for index, cell in enumerate(array):
            if isinstance(cell, (int, float)):
                self.xlsx_sheet.write_number(self.current_row, index, cell)
            elif isinstance(cell, bool):
                self.xlsx_sheet.write_boolean(self.current_row, index, cell)
            elif isinstance(cell, datetime):
                self.xlsx_sheet.write_datetime(
                    self.current_row,
                    index,
                    cell.year,
                    cell.month,
                    cell.day,
                    cell.hour,
                    cell.minute,
                    cell.second,
                )
            elif isinstance(cell, date):
                self.xlsx_sheet.write_datetime(
                    self.current_row,
                    index,
                    cell.year,
                    cell.month,
                    cell.day,
                    0,
                    0,
                    0,
                )
            elif isinstance(cell, time):
                self.xlsx_sheet.write_datetime(
                    self.current_row,
                    index,
                    0,
                    0,
                    0,
                    cell.hour,
                    cell.minute,
                    cell.second,
                )
            else:
                self.xlsx_sheet.write_string(
                    self.current_row, index, str(cell)
                )

        self.current_row += 1

    def close(self):
        pass


class XLSXWriter(IWriter):
    """
    xlsx writer
    """

    def __init__(
        self,
        file_name,
        file_type,
        constant_memory=True,
        default_date_format="dd/mm/yy",
        **keywords
    ):
        """
        Open a file for writing

        Please note that this writer configure xlsxwriter's BookWriter to use
        constant_memory by default.

        :param keywords: **default_date_format** control the date time format.
                         **constant_memory** if true, reduces the memory
                         footprint when writing large files. Other parameters
                         can be found in `xlsxwriter's documentation
                         <http://xlsxwriter.readthedocs.io/workbook.html>`_
        """
        if "single_sheet_in_book" in keywords:
            keywords.pop("single_sheet_in_book")
        self.xlsx_book = Book()
        self.xlsx_book.open(file_name)

    def create_sheet(self, name):
        sheet = self.xlsx_book.add_worksheet(name)
        return XLSXSheetWriter(sheet)

    def close(self):
        """
        This call actually save the file
        """
        self.xlsx_book.close()
        self.xlsx_book = None



import os
import datetime

import pyexcel as pe


class TestDateFormat:
    def test_writing_date_format(self):
        excel_filename = "testdateformat.xlsx"
        data = [
            [
                datetime.date(2014, 12, 25),
                datetime.time(11, 11, 11),
                datetime.datetime(2014, 12, 25, 11, 11, 11),
            ]
        ]
        pe.save_as(
            dest_file_name=excel_filename,
            array=data,
            library="pyexcel-libxlsxw",
        )
        r = pe.get_sheet(file_name=excel_filename, library="pyexcel-xls")
        assert isinstance(r[0, 0], datetime.date) is True
        assert r[0, 0].strftime("%d/%m/%y") == "25/12/14"
        assert isinstance(r[0, 1], datetime.time) is True
        assert r[0, 1].strftime("%H:%M:%S") == "11:11:11"
        assert isinstance(r[0, 2], datetime.date) is True
        assert r[0, 2].strftime("%d/%m/%y %H:%M:%S") == "25/12/14 11:11:11"
        os.unlink(excel_filename)



import pyexcel

from nose.tools import eq_


class TestStringIO:
    def test_xls_output_stringio(self):
        data = [[1, 2, 3], [4, 5, 6]]
        io = pyexcel.save_as(
            dest_file_type="xlsx", array=data, library="pyexcel-libxlsxw"
        )
        r = pyexcel.get_sheet(file_type="xlsx", file_content=io.getvalue())
        result = [1, 2, 3, 4, 5, 6]
        actual = list(r.enumerate())
        eq_(result, actual)



import os
from datetime import date, time, datetime

from base import PyexcelWriterBase, PyexcelHatWriterBase
from pyexcel_xls import get_data
from pyexcel_libxlsxw import xlsxw as xlsx

from nose.tools import eq_


class TestNativeXLWriter:
    def setUp(self):
        self.testfile = "xlwriter.xlsx"

    def test_write_book(self):
        self.content = {
            "Sheet1": [[1, 1, 1, 1], [2, 2, 2, 2], [3, 3, 3, 3]],
            "Sheet2": [[4, 4, 4, 4], [5, 5, 5, 5], [6, 6, 6, 6]],
            "Sheet3": [[u"X", u"Y", u"Z"], [1, 4, 7], [2, 5, 8], [3, 6, 9]],
        }
        writer = xlsx.XLSXWriter(self.testfile, "xlsx")
        writer.write(self.content)
        writer.close()
        content = get_data(self.testfile)
        for key in content.keys():
            content[key] = list(content[key])
        eq_(content, self.content)

    def test_write_dates(self):
        self.content = {
            "date": [[date(2020, 10, 11)]],
            "time": [[time(11, 22, 11)]],
            "datetime": [[datetime(2020, 11, 11, 12, 12, 12)]],
        }
        writer = xlsx.XLSXWriter(self.testfile, "xlsx")
        writer.write(self.content)
        writer.close()
        content = get_data(self.testfile)
        for key in content.keys():
            content[key] = list(content[key])
        eq_(content, self.content)

    def tearDown(self):
        if os.path.exists(self.testfile):
            os.unlink(self.testfile)


class TestXLSnCSVWriter(PyexcelWriterBase):
    def setUp(self):
        self.testfile = "test.xlsx"
        self.testfile2 = "test.csv"

    def tearDown(self):
        if os.path.exists(self.testfile):
            os.unlink(self.testfile)
        if os.path.exists(self.testfile2):
            os.unlink(self.testfile2)


class TestXLSHatWriter(PyexcelHatWriterBase):
    def setUp(self):
        self.testfile = "test.xlsx"

    def tearDown(self):
        if os.path.exists(self.testfile):
            os.unlink(self.testfile)
